@@ -58,29 +58,6 @@ We don't currently support these modules, but would like to::
     import tkinter
     import pickle     # should (optionally) bring in cPickle on Python 2
 
-
-Notes
------
-This module only supports Python 2.6, Python 2.7, and Python 3.1+.
-
-The following renames are already supported on Python 2.7 without any
-additional work from us::
-
-    reload() -> imp.reload()
-    reduce() -> functools.reduce()
-    StringIO.StringIO -> io.StringIO
-    Bytes.BytesIO -> io.BytesIO
-
-Old things that can one day be fixed automatically by futurize.py::
-
-  string.uppercase -> string.ascii_uppercase   # works on either Py2.7 or Py3+
-  sys.maxint -> sys.maxsize      # but this isn't identical
-
-TODO: Check out these:
-Not available on Py2.6:
-  unittest2 -> unittest?
-  buffer -> memoryview?
-
 """
 
 from __future__ import absolute_import, division, print_function
@@ -92,7 +69,6 @@ import contextlib
 import types
 import copy
 import os
-import importlib
 
 from future.utils import PY2, PY3
 
@@ -719,6 +695,10 @@ def import_(module_name, backport=False):
         >>> from future.backports import http.client
 
     """
+    # Python 2.6 doesn't have importlib in the stdlib, so it requires
+    # the backported ``importlib`` package from PyPI as a dependency to use
+    # this function:
+    import importlib
 
     if PY3:
         return __import__(module_name)
