@@ -7,8 +7,8 @@ from __future__ import absolute_import, unicode_literals, print_function
 import sys
 from future.builtins import *
 from future.utils import (old_div, istext, isbytes, native, PY2, PY3,
-                         native_str, raise_)
-
+                         native_str, raise_, as_native_str, ensure_new_type,
+                         bytes_to_native_str)
 
 from numbers import Integral
 from future.tests.base import unittest, skip26
@@ -105,7 +105,6 @@ class TestUtils(unittest.TestCase):
         self.assertFalse(isbytes(self.s2))
 
     @skip26
-    @unittest.expectedFailure
     def test_raise_(self):
         """
         The with_value() test currently fails on Py3
@@ -143,6 +142,52 @@ class TestUtils(unittest.TestCase):
             self.assertEqual(str(e), "An error")
 
 
+    def test_as_native_str(self):
+        """
+        Tests the decorator as_native_str()
+        """
+        class MyClass(object):
+            @as_native_str()
+            def __repr__(self):
+                return u'abc'
+            
+        obj = MyClass()
+       
+        self.assertEqual(repr(obj), 'abc')
+        if PY2:
+            self.assertEqual(repr(obj), b'abc')
+        else:
+            self.assertEqual(repr(obj), u'abc')
+
+    def test_ensure_new_type(self):
+        s = u'abcd'
+        s2 = str(s)
+        self.assertEqual(ensure_new_type(s), s2)
+        self.assertEqual(type(ensure_new_type(s)), str)
+
+        b = b'xyz'
+        b2 = bytes(b)
+        self.assertEqual(ensure_new_type(b), b2)
+        self.assertEqual(type(ensure_new_type(b)), bytes)
+
+        i = 10000000000000
+        i2 = int(i)
+        self.assertEqual(ensure_new_type(i), i2)
+        self.assertEqual(type(ensure_new_type(i)), int)
+
+    def test_bytes_to_native_str(self):
+        """
+        Test for issue #47
+        """
+        b = bytes(b'abc')
+        s = bytes_to_native_str(b)
+        if PY2:
+            self.assertEqual(s, b)
+        else:
+            self.assertEqual(s, 'abc')
+        self.assertTrue(isinstance(s, native_str))
+        self.assertEqual(type(s), native_str)
+
 
 if __name__ == '__main__':
     unittest.main()
